local FOLDER_NAME, private = ...
local core = LibStub("AceAddon-3.0"):NewAddon(FOLDER_NAME, "AceEvent-3.0")
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(FOLDER_NAME)
local strformat, pairs, strupper, select, type, strsplit, match = string.format, pairs, strupper, select, type, strsplit, strmatch
local UnitName, UnitClass, GetItemCount, GetItemIcon, GetNumGuildBankTabs, GetGuildBankTabInfo, GetGuildBankItemInfo, GetGuildBankItemLink, IsInGuild, GetItemInfo = UnitName, UnitClass, GetItemCount, GetItemIcon, GetNumGuildBankTabs, GetGuildBankTabInfo, GetGuildBankItemInfo, GetGuildBankItemLink, IsInGuild, GetItemInfo
local charsList,  slotItems= {}, {}
local playerName, playerClass = UnitName("player"), strupper(select(2,UnitClass("player")))
local MAX_GUILDBANK_SLOTS_PER_TAB = 98
local scanPending

local defaults = {
    profile = {
        modules = {['*'] = false},
        tracking = {['*'] = false},
        showShort = false,
        showIcons = false,
        iconsOnly = false,
        tooltipIcons = false,
        tooltipIconsOnly = false,
        tooltipShowShort = false,
        lineColor = true,
        textColor = true,
        showZero = true,
        showBank = true,
        showAlts = true,
        showGuild = true,
        scale = 1,
        scroll = 900,
    },
    realm = {
        chars = {
            ['*'] = {
                data = {
                    ['*'] = {inv = 0, bank = 0, guild = 0},
                },
            }
        }
    }
}

function core:OnInitialize()
    self.charName = playerName
    self.db = LibStub("AceDB-3.0"):New(FOLDER_NAME.."_DB", defaults, true)
    if not self.db.realm.chars[self.charName].class then
        self.db.realm.chars[self.charName].class = playerClass
    end
    for name in pairs(self.db.realm.chars) do
        charsList[name] = name
    end
    self:SetupOptions()
end

private.charsList = charsList
private.scanPending = scanPending

function core:countInv()
    for k,itemid in pairs (private.vanhides.id) do
        self:countSave(itemid)
    end
    for k,itemid in pairs (private.bchides.id) do
        self:countSave(itemid)
    end
    for k,itemid in pairs (private.lkhides.id) do
        self:countSave(itemid)
    end
    for k,itemid in pairs (private.catahides.id) do
        self:countSave(itemid)
    end
    for k,itemid in pairs (private.mophides.id) do
        self:countSave(itemid)
    end
    for k,itemid in pairs (private.wodhides.id) do
        self:countSave(itemid)
    end
    self:applySettings()
end

function core:countGuild()
    for k,itemid in pairs (private.vanhides.id) do
        self:guildCounts(itemid)
    end
    for k,itemid in pairs (private.bchides.id) do
        self:guildCounts(itemid)
    end
    for k,itemid in pairs (private.lkhides.id) do
        self:guildCounts(itemid)
    end
    for k,itemid in pairs (private.catahides.id) do
        self:guildCounts(itemid)
    end
    for k,itemid in pairs (private.mophides.id) do
        self:guildCounts(itemid)
    end
    for k,itemid in pairs (private.wodhides.id) do
        self:guildCounts(itemid)
    end
    private.scanPending = false
    core:applySettings()
end

function core:OnEnable()
    self.db.RegisterCallback(self, "OnProfileChanged", "applySettings")
    self.db.RegisterCallback(self, "OnProfileCopied", "applySettings")
    self.db.RegisterCallback(self, "OnProfileReset", "applySettings")
    self:countInv()
    core:RegisterEvent("BAG_UPDATE", "countInv")
    core:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED", "scanGuild")
    core:RegisterEvent("GUILDBANKFRAME_CLOSED", "countGuild")
end

function core:OnDisable()
    core:UnregisterAllEvents()
end

function core:applySettings()
    core:onTextUpdate()
end

function core:scanGuild()
    if not IsInGuild() then return end
    private.scanPending = true
    core:applySettings()
    local numTabs = GetNumGuildBankTabs()
    local index = 0
    slotItems = {}
    for tab = 1, numTabs do
        local name, icon, isViewable, canDeposit, numWithdrawals, remainingWithdrawals = GetGuildBankTabInfo(tab)
        if isViewable then
            for slot = 1, MAX_GUILDBANK_SLOTS_PER_TAB do
                local link = GetGuildBankItemLink(tab, slot)
                if link and self:linkMatch(link) then
                    index = index + 1
                    local _, count = GetGuildBankItemInfo(tab, slot)
                    count = (count > 1 and count) or nil
                    if count then
                        slotItems[index] = strformat('%s,%d', self:linkMatch(link), count)
                    else
                        slotItems[index] = self:linkMatch(link)
                    end
                end
            end
        end
    end
end

function core:guildCounts(itemid)
    local total = 0
    local name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemid)
    local id = self:linkMatch(link)
    for index, item in pairs(slotItems) do
        local slotlink, slotcount = strsplit(',', item)
        if slotlink and slotlink == id then
            total = total + (slotcount or 1)
        end
    end
    self.db.realm.chars[self.charName].data[itemid].guild = total or 0
end

function core:linkMatch(link)
    if not link then return nil end
    return link:match('|Hitem:(%d+):[%d:]+|h%[(.-)%]|h')
end

function core:countSave(itemid)
    local itemid = itemid
    local inv = GetItemCount(itemid)
    local bank = GetItemCount(itemid, true)
    bank = bank - inv
    self.db.realm.chars[self.charName].data[itemid].inv = inv or 0
    self.db.realm.chars[self.charName].data[itemid].bank = bank or 0
end

function core:CharHasElement(itemid)
    if self.db.realm.chars[self.charName].data[itemid].inv > 0 or (self.db.profile.showBank and self.db.realm.chars[self.charName].data[itemid].bank > 0) or (self.db.profile.showGuild and self.db.realm.chars[self.charName].data[itemid].guild > 0)then
        return true
    end
    if self.db.profile.showAlts then
        for char, _ in pairs(self.db.realm.chars) do
            if self.db.realm.chars[char].data[itemid].inv > 0 or (self.db.profile.showBank and self.db.realm.chars[char].data[itemid].bank > 0) or (self.db.profile.showGuild and self.db.realm.chars[char].data[itemid].guild > 0) then
                return true
            end
        end
    end
    return false
end

function core:InvCountString(itemid, char)
    local tempChar = self.db.realm.chars[self.charName]
    if char then
        tempChar = self.db.realm.chars[char]
    end
    return self:Color(tempChar.data[itemid].inv)
end

function core:BankCountString(itemid, char)
    local tempChar = self.db.realm.chars[self.charName]
    if char then
        tempChar = self.db.realm.chars[char]
    end
    return self:Color(tempChar.data[itemid].bank)
end

function core:GuildCountString(itemid, char)
    local tempChar = self.db.realm.chars[self.charName]
    if char then
        tempChar = self.db.realm.chars[char]
    end
    return self:Color(tempChar.data[itemid].guild)
end

function core:Color(count)
    if core.db.profile.textColor then
        local perc = 0
        local max = 10
        if max ~= 0 then
            perc = count / max
        end
        local r1, g1, b1
        local r2, g2, b2
        if perc <= 0.5 then
            perc = perc * 2
            r1, g1, b1 = 1, 0, 0
            r2, g2, b2 = 1, 1, 0
        else
            perc = perc * 2 - 1
            r1, g1, b1 = 1, 1, 0
            r2, g2, b2 = 0, 1, 0
        end
        local r, g, b = r1 + (r2 - r1)*perc, g1 + (g2 - g1)*perc, b1 + (b2 - b1)*perc
        if r < 0 then
            r = 0
        elseif r > 1 then
            r = 1
        end
        if g < 0 then
            g = 0
        elseif g > 1 then
            g = 1
        end
        if b < 0 then
            b = 0
        elseif b > 1 then
            b = 1
        end
        r, g, b =  r * 255, g * 255, b * 255
        color = strformat("%02x%02x%02x", r, g, b)
    else
        color = "FFFFFF"
    end
    count = ("|cff"..color..count.."|r")
    return count
end